@@ -1,10 +1,18 @@
 import sys, os, builtins, inspect
 from importlib import machinery
+import zipimport
 
 def _loadmodule(name, path):
-    module = machinery.SourceFileLoader(name, path).load_module()
-    globals()[name] = module
-    return module
+    if path.rsplit('.',1)[-1] == 'zip':
+        zpackage = zipimport.zipimporter(path)
+        globals()[name] = zpackage
+        sys.modules[name] = zpackage
+        return zpackage
+    else:
+        module = machinery.SourceFileLoader(name, path).load_module()
+        globals()[name] = module
+        sys.modules[name] = module
+        return module
 
 def _folderwalker(cwd, down_only=False):
     cwd = cwd.replace('\\','/').rstrip('/')
@@ -23,15 +31,21 @@ def _folderwalker(cwd, down_only=False):
                 del dirs[i]
         yield root.replace('\\','/')
 
-def _find(cwd, name, down_only=False):
+def _find(cwd, name, down_only=False, zip=False):
     for root in _folderwalker(cwd, down_only=down_only):
         path = os.path.join(root, name).replace('\\','/')
         ppath = os.path.join(path, '__init__.py').replace('\\','/')
-        path = path+'.py'
-        if os.path.exists(path):
-            return path
-        if os.path.exists(ppath):
+        pppath = os.path.join(root, '__init__.py').replace('\\','/')
+        zpath = path+'.zip'
+        path += '.py'
+        if os.path.exists(pppath) and os.path.split(root)[1] == name:
+            return pppath
+        elif os.path.exists(ppath):
             return ppath
+        elif os.path.exists(path):
+            return path
+        elif zip and os.path.exists(zpath):
+            return zpath            
     else:
         raise ImportError('FooFinder could not find '+name+' searching from '+cwd)
 
@@ -42,7 +56,7 @@ def _is_interactive():
     return hasattr(sys, 'ps1')
     
 def _import(pname, *args, **kwargs):
-    if 'FooFinder' not in pname or not args[2]:
+    if 'FooFinder' not in pname or len(args)< 3 or not args[2]:
         return globals()['_original_import'](pname, *args, **kwargs)
     name = args[2][0]
     if name in globals():
@@ -50,7 +64,9 @@ def _import(pname, *args, **kwargs):
     if 'frame' in kwargs:
         frame = kwargs['frame']
     else:
-        frame = inspect.currentframe().f_back
+        frame = inspect.currentframe().f_back    
+        if inspect.getframeinfo(frame).function == '_import': #Not sure why this happens
+            frame = frame.f_back #Fix for strange occurence in anaconda install where it kept returning _import
     if _is_ipython() or _is_interactive():
         cwd = os.getcwd()
     else:
@@ -58,9 +74,18 @@ def _import(pname, *args, **kwargs):
     spname = pname.rsplit('.',1)[-1]
     if spname != 'FooFinder': #relative child imports
         if spname not in globals():
-            packpath = _find(cwd, spname)
-            _loadmodule(pname, packpath)
-        package = globals()[pname]
+            packpath = _find(cwd, spname, zip=True)
+            package = _loadmodule(spname, packpath)
+            if packpath.rsplit('.',1)[-1] == 'zip':
+                zmodule = package.load_module(name)
+                class zpackage:
+                    zpackage = package
+                setattr(zpackage, name, zmodule)
+                package = zpackage
+            globals()[pname] = package #_original_import support
+            sys.modules[pname] = package #_original_import support
+        else:
+            package = globals()[spname]
         if not hasattr(package, name):
             cwd = packpath.rsplit('/',1)[0]
             path = _find(cwd, name, down_only=True)
